//  Copyright 2021 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit
import KakaoPartnerSDKFriend

class PartnerSDKFriendPickerSettingViewController : UIViewController {
    
    var safeAreaLayoutGuide: UILayoutGuide!
    var tableView: UITableView!
    var bottomView: UIView!
    var cancelBtn: UIButton!
    var okBtn: UIButton!
    let viewModel = PartnerSDKFriendPickerSettingViewModel()

    
    override func loadView() {
        super.loadView()
        
        self.safeAreaLayoutGuide = view.layoutMarginsGuide
    
        self.tableView = UITableView(frame: self.view.frame)
        self.tableView.dataSource = self
        self.tableView.delegate = self
        self.tableView.separatorStyle = .none
        self.tableView.register(PartnerSDKFriendPickerSettingCell.self, forCellReuseIdentifier: "cell")
        self.view.addSubview(self.tableView)

        self.bottomView = UIView(frame: .zero)
        self.view.addSubview(self.bottomView)

        self.cancelBtn = UIButton(frame: .zero)
        self.cancelBtn.setTitle("취소", for: .normal)
        self.cancelBtn.titleLabel?.font = UIFont.systemFont(ofSize: 16, weight: .bold)
        self.cancelBtn.backgroundColor = .red
        self.cancelBtn.setTitleColor(.black, for: .normal)
        self.cancelBtn.addTarget(self, action: #selector(onCancelBtn), for: .touchUpInside)

        self.okBtn = UIButton(frame: .zero)
        self.okBtn.setTitle("피커 생성", for: .normal)
        self.okBtn.titleLabel?.font = UIFont.systemFont(ofSize: 16, weight: .bold)
        self.okBtn.backgroundColor = .green
        self.okBtn.setTitleColor(.black, for: .normal)
        self.okBtn.addTarget(self, action: #selector(onOkBtn), for: .touchUpInside)

        self.bottomView.addSubview(self.cancelBtn)
        self.bottomView.addSubview(self.okBtn)
        
        self.view.backgroundColor = .white
    }
    
    public override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        self.tableView.translatesAutoresizingMaskIntoConstraints = false
        self.tableView.topAnchor.constraint(equalTo: self.safeAreaLayoutGuide.topAnchor).isActive = true
        self.tableView.leftAnchor.constraint(equalTo: self.view.leftAnchor).isActive = true
        self.tableView.bottomAnchor.constraint(equalTo: self.bottomView.topAnchor).isActive = true
        self.tableView.rightAnchor.constraint(equalTo: self.view.rightAnchor).isActive = true

        self.bottomView.translatesAutoresizingMaskIntoConstraints = false
        self.bottomView.topAnchor.constraint(equalTo: self.tableView.bottomAnchor).isActive = true
        self.bottomView.leftAnchor.constraint(equalTo: self.view.leftAnchor).isActive = true
        self.bottomView.bottomAnchor.constraint(equalTo: self.safeAreaLayoutGuide.bottomAnchor).isActive = true
        self.bottomView.rightAnchor.constraint(equalTo: self.view.rightAnchor).isActive = true
        self.bottomView.heightAnchor.constraint(equalToConstant: 60).isActive = true

        self.cancelBtn.translatesAutoresizingMaskIntoConstraints = false
        self.cancelBtn.topAnchor.constraint(equalTo: self.bottomView.topAnchor).isActive = true
        self.cancelBtn.leftAnchor.constraint(equalTo: self.bottomView.leftAnchor).isActive = true
        self.cancelBtn.bottomAnchor.constraint(equalTo: self.bottomView.bottomAnchor).isActive = true
        self.cancelBtn.widthAnchor.constraint(equalTo: self.bottomView.widthAnchor, multiplier: 0.5).isActive = true

        self.okBtn.translatesAutoresizingMaskIntoConstraints = false
        self.okBtn.topAnchor.constraint(equalTo: self.bottomView.topAnchor).isActive = true
        self.okBtn.rightAnchor.constraint(equalTo: self.bottomView.rightAnchor).isActive = true
        self.okBtn.bottomAnchor.constraint(equalTo: self.bottomView.bottomAnchor).isActive = true
        self.okBtn.widthAnchor.constraint(equalTo: self.bottomView.widthAnchor, multiplier: 0.5).isActive = true
    }
    
    @objc func onCancelBtn() {
        dismiss(animated: true, completion: nil)
    }
    
    @objc func onOkBtn() {
        viewModel.printSnapShot()

        let param = PickerRequestParams(title: viewModel.pickerTitle,
                                        pickerFriendOrder: viewModel.friendOrder,
                                        pickerFriendType: viewModel.friendType,
                                        pickerFriendFilter: viewModel.friendFilter,
                                        viewApperance: viewModel.viewApperance,
                                        countryCodes: viewModel.countryCodes,
                                        pickerUsingOsFilter: viewModel.friendOS,
                                        orientation: viewModel.orientation,
                                        enableSearch: viewModel.isSearchFriendEnabled,
                                        searchPlaceHolderText: viewModel.searchPlaceHolder,
                                        showMyProfile: viewModel.isMyProfileEnabled,
                                        showFavorite: viewModel.isFavoriteEnabled,
                                        disableSelectReasons: viewModel.disabledSelectReason == nil ?
                                            nil : [viewModel.disabledSelectReason!],
                                        displayAllProfile: viewModel.displayAllProfile, showPickedFriend: viewModel.isSelectedFriendsEnabled,
                                        maxPickableCount: viewModel.maxPickableCount,
                                        minPickableCount: viewModel.minPickableCount)
        
        let completion : (SelectedUsers?, Error?) -> Void = { selectedUsers, error in
            if let error = error {
                let alert = UIAlertController(title: "Error", message: "\(error)", preferredStyle: UIAlertController.Style.alert)
                let okAction = UIAlertAction(title: "OK", style: .default)
                alert.addAction(okAction)
                
                self.present(alert, animated: false, completion: nil)
                print(error)
            }
            else {
                var message = ""
                if let selectedUsers = selectedUsers {
                    selectedUsers.users?.enumerated().forEach {
                        message += "nickname: \($0.element.profileNickname ?? "")\n uuid: \($0.element.uuid) \n"
                    }
                }
                else {
                    message = "No Friends"
                }
                
                let alert = UIAlertController(title: "Success", message: "\(message)", preferredStyle: UIAlertController.Style.alert)
                let okAction = UIAlertAction(title: "OK", style: .default)
                alert.addAction(okAction)
                
                self.present(alert, animated: false, completion: nil)
                print(selectedUsers ?? "")
            }
        }
        
        if viewModel.isSinglePicker {
            if viewModel.isPopupPicker {
                FriendPicker.shared.selectSinglePopup(requestParams: param, completion: completion)
            }
            else {
                FriendPicker.shared.selectSingle(requestParams: param, completion: completion)
            }
        }
        else {
            if viewModel.isPopupPicker {
                FriendPicker.shared.selectMultiplePopup(requestParams: param, completion: completion)
            }
            else {
                FriendPicker.shared.selectMultiple(requestParams: param, completion: completion)
            }
        }
    }
}


extension PartnerSDKFriendPickerSettingViewController: UITableViewDelegate {
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        view.endEditing(true)
    }
}

extension PartnerSDKFriendPickerSettingViewController: UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return viewModel.numberOfItems
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if let cell = tableView.dequeueReusableCell(withIdentifier: "cell") as? PartnerSDKFriendPickerSettingCell {
            cell.item = viewModel.items[indexPath.item]
            cell.backgroundColor = .white
            cell.selectionStyle = .none
            cell.update()
            return cell
        }

        return PartnerSDKFriendPickerSettingCell()
    }
    
    public func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 30
    }
}


class PartnerSDKFriendPickerSettingCell : UITableViewCell {
    
    var parentViewController: UIViewController? {
        var parentResponder: UIResponder? = self
        while parentResponder != nil {
            parentResponder = parentResponder!.next
            if let viewController = parentResponder as? UIViewController {
                return viewController
            }
        }
        return nil
    }
    
    var item: Item?
    var checkBox: UIButton?
    var inputBox: UITextField?
    var menuBox: UIButton?
    
    override func prepareForReuse() {
        super.prepareForReuse()
        for view in contentView.subviews {
            view.removeFromSuperview()
        }
    }
    
    func update() {
        guard let item = item else { return }
        
        let titleLabel = UILabel(frame: .zero)
        titleLabel.text = item.title
        titleLabel.textColor = .black
        titleLabel.sizeToFit()
        titleLabel.font = UIFont.systemFont(ofSize: 16, weight: .semibold)
        self.contentView.addSubview(titleLabel)
        
        titleLabel.translatesAutoresizingMaskIntoConstraints = false
        titleLabel.leftAnchor.constraint(equalTo: self.contentView.leftAnchor, constant: 20).isActive = true
        titleLabel.widthAnchor.constraint(equalToConstant: titleLabel.bounds.width).isActive = true
        titleLabel.centerYAnchor.constraint(equalTo:self.contentView.centerYAnchor).isActive = true

        if let it = (item as? CheckBoxItem) {
            checkBox = UIButton(frame: .zero)
            checkBox?.layer.cornerRadius = 15
            checkBox?.layer.masksToBounds = true
            checkBox?.layer.borderWidth = 1.0
            checkBox?.layer.borderColor = UIColor.black.cgColor
            checkBox?.setBackgroundImage(it.value == true ? UIImage(named: "checked") : nil, for: .normal)
            checkBox?.addTarget(self, action: #selector(handleCheckBox), for: .touchUpInside)

            self.contentView.addSubview(checkBox!)
            checkBox?.translatesAutoresizingMaskIntoConstraints = false
            checkBox?.rightAnchor.constraint(equalTo: self.contentView.rightAnchor, constant: -30).isActive = true
            checkBox?.centerYAnchor.constraint(equalTo:self.contentView.centerYAnchor).isActive = true
            checkBox?.widthAnchor.constraint(equalToConstant: 29).isActive = true
            checkBox?.heightAnchor.constraint(equalToConstant: 29).isActive = true
        }
        else if let it = (item as? InputBoxItem) {
            inputBox = UITextField(frame: .zero)
            if it.type == .numeric { inputBox?.keyboardType = .numberPad}
            inputBox?.backgroundColor = .white
            inputBox?.layer.cornerRadius = 5
            inputBox?.layer.masksToBounds = true
            inputBox?.text = it.value
            inputBox?.textAlignment = .center
            inputBox?.autocorrectionType = .no
            inputBox?.textColor = .black
            inputBox?.backgroundColor = #colorLiteral(red: 0.8977650404, green: 0.8979157209, blue: 0.8977451921, alpha: 1)
            inputBox?.attributedPlaceholder = NSAttributedString(string: it.placeHolder ?? "",
                                                                attributes: [NSAttributedString.Key.foregroundColor: UIColor.darkGray])
            inputBox?.layer.borderWidth = 0.5
            inputBox?.layer.borderColor = UIColor.black.cgColor
            inputBox?.delegate = self

            self.contentView.addSubview(inputBox!)
            inputBox?.translatesAutoresizingMaskIntoConstraints = false
            inputBox?.rightAnchor.constraint(equalTo: self.contentView.rightAnchor, constant: -30).isActive = true
            inputBox?.leftAnchor.constraint(equalTo: titleLabel.rightAnchor, constant: 30).isActive = true
            inputBox?.centerYAnchor.constraint(equalTo:self.contentView.centerYAnchor).isActive = true
            inputBox?.heightAnchor.constraint(equalToConstant: 29).isActive = true
        }
        else if let it = (item as? MenuBoxItem) {

            menuBox = UIButton(frame: .zero)
            menuBox?.layer.borderWidth = 0.5
            menuBox?.layer.borderColor = UIColor.black.cgColor
            menuBox?.layer.cornerRadius = 5
            menuBox?.layer.masksToBounds = true
            menuBox?.setTitleColor(UIColor.black, for: .normal)
            menuBox?.setTitle(it.value, for: .normal)
            menuBox?.backgroundColor = #colorLiteral(red: 0.8977650404, green: 0.8979157209, blue: 0.8977451921, alpha: 1)
            
            menuBox?.addTarget(self, action: #selector(handleMenuBox), for: .touchUpInside)
            
            self.contentView.addSubview(menuBox!)
            menuBox?.translatesAutoresizingMaskIntoConstraints = false
            menuBox?.rightAnchor.constraint(equalTo: self.contentView.rightAnchor, constant: -30).isActive = true
            menuBox?.centerYAnchor.constraint(equalTo:self.contentView.centerYAnchor).isActive = true
            menuBox?.widthAnchor.constraint(equalToConstant: 150).isActive = true
            menuBox?.heightAnchor.constraint(equalToConstant: 29).isActive = true
        }
        
    }
    
    @objc func handleCheckBox() {
        if let it = item as? CheckBoxItem {
            let enable = it.action?() ?? false
            checkBox?.setBackgroundImage(enable == true ? UIImage(named: "checked") : nil, for: .normal)
            it.value = enable
        }
    }
    
    @objc func handleMenuBox() {
        if let it = item as? MenuBoxItem {
            if let vc = self.parentViewController {
                
                let alert = UIAlertController(title: nil, message: nil, preferredStyle: .actionSheet)
                
                for menu in it.menu {
                    alert.addAction(UIAlertAction(title: menu, style: .default) { _ in
                        self.menuBox?.setTitle(menu, for: .normal)
                        let selectedMenu = it.action?(menu)
                        it.value = selectedMenu
                    })
                }
                
                let cancel = UIAlertAction(title: "CANCEL", style: .destructive, handler: nil)
                alert.addAction(cancel)
                vc.present(alert, animated: true)
            }
        }
    }
}

extension PartnerSDKFriendPickerSettingCell: UITextFieldDelegate {
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        if let it = item as? InputBoxItem {
            it.action?(textField.text)
            it.value = textField.text
        }
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.endEditing(true)
        return true
    }
}

// ViewModel
class PartnerSDKFriendPickerSettingViewModel {
    
    var items: [Item] = []
    
    // Setting Value
    var isSinglePicker = false
    var isPopupPicker = false
    var pickerTitle: String = "카카오톡 친구 선택"
    var friendOrder: PickerFriendOrder?
    var friendType: PickerFriendType?
    var friendFilter: PickerFriendFilter?
    var viewApperance: ViewApperance?
    var countryCodes: [String]?
    var friendOS: PickerUsingOsFilter?
    var orientation: PickerOrientation?
    var isSearchFriendEnabled: Bool = true
    var isIndexViewEnabled: Bool = true
    var searchPlaceHolder: String?
    var isMyProfileEnabled: Bool = true
    var isFavoriteEnabled: Bool = true
    var disabledSelectReason: DisableSelectReason?
    var maxPickableCount: Int = 30
    var minPickableCount: Int = 1
    var isSelectedFriendsEnabled = true
    var displayAllProfile: Bool = false
    ///
    var numberOfItems: Int {
        return items.count
    }
    
    init() {
        
        items.append(CheckBoxItem(title: "싱글 피커", action: {
            self.isSinglePicker = !self.isSinglePicker
            return self.isSinglePicker
        }))
        items.append(CheckBoxItem(title: "팝업 피커", action: {
            self.isPopupPicker = !self.isPopupPicker
            return self.isPopupPicker
        }))
        items.append(InputBoxItem(title: "타이틀", placeHolder: "카카오톡 친구 선택", action: { pickerTitle in
            self.pickerTitle = pickerTitle ?? "카카오톡 친구 선택"
        }))
        
        items.append(MenuBoxItem(title: "친구 정렬", menu: ["NULL","AGE"], action: { type in
            switch type {
            case "AGE":
                self.friendOrder = .age
            default:
                self.friendOrder = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "친구 타입", menu: ["NULL","TALK","STORY","TALKSTORY"], action: { type in
            switch type {
            case "TALK":
                self.friendType = .talk
            case "STORY":
                self.friendType = .story
            case "TALKSTORY":
                self.friendType = .talkstory
            default:
                self.friendType = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "친구 필터", menu: ["NULL","REGISTERED","INVITABLE","NONE"], action: { type in
            switch type {
            case "REGISTERED":
                self.friendFilter = .registered
            case "INVITABLE":
                self.friendFilter = .invitable
            case "NONE":
                self.friendFilter = PickerFriendFilter.none
            default:
                self.friendFilter = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "Light/Dark 모드", menu: ["NULL","AUTO","LIGHT","DARK"], action: { type in
            switch type {
            case "AUTO":
                self.viewApperance = .auto
            case "LIGHT":
                self.viewApperance = .light
            case "DARK":
                self.viewApperance = .dark
            default:
                self.viewApperance = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(InputBoxItem(title: "국가 코드", placeHolder: "kr,jp,us,....", action: { countryCodes in
            self.countryCodes = countryCodes?.split(separator: ",").map{it in String(it)}
        }))
        
        items.append(InputBoxItem(title: "검색창 PlaceHolder", placeHolder: "이름, 닉네임 검색", action: { placeholder in
            self.searchPlaceHolder = placeholder
        }))
        
        items.append(MenuBoxItem(title: "친구 디바이스 OS", menu: ["NULL","ALL","ANDROID","iOS"], action: { type in
            switch type {
            case "ALL":
                self.friendOS = .all
            case "ANDROID":
                self.friendOS = .android
            case "iOS":
                self.friendOS = .ios
            default:
                self.friendOS = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "피커 Orientation", menu: ["NULL","AUTO","PORTRAIT","LANDSCAPE"], action: { type in
            switch type {
            case "AUTO":
                self.orientation = .auto
            case "PORTRAIT":
                self.orientation = .portrait
            case "LANDSCAPE":
                self.orientation = .landscape
            default:
                self.orientation = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(CheckBoxItem(title: "검색 허용", value: true, action: {
            self.isSearchFriendEnabled = !self.isSearchFriendEnabled
            return self.isSearchFriendEnabled
        }))
        
        items.append(CheckBoxItem(title: "인덱스 허용", value: true, action: {
            self.isIndexViewEnabled = !self.isIndexViewEnabled
            return self.isIndexViewEnabled
        }))
        
        items.append(CheckBoxItem(title: "내 프로필 보기", value: true, action: {
            self.isMyProfileEnabled = !self.isMyProfileEnabled
            return self.isMyProfileEnabled
        }))

        items.append(CheckBoxItem(title: "내 즐겨찾기 보기", value: true, action: {
            self.isFavoriteEnabled = !self.isFavoriteEnabled
            return self.isFavoriteEnabled
        }))
        
        items.append(MenuBoxItem(title: "친구 비활성화 옵션", menu: ["NULL","MSG_BLOCKED","ETC"], action: { type in
            switch type {
            case "MSG_BLOCKED":
                self.disabledSelectReason = .msgBlocked
            case "ETC":
                self.disabledSelectReason = .none
            default:
                self.disabledSelectReason = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(InputBoxItem(title: "최대 선택 수", placeHolder: "30", type: .numeric,action: { count in
            if let count = count {
                if count.isEmpty {
                    self.maxPickableCount = 30
                }
                else {
                    self.maxPickableCount = Int(count) ??  self.maxPickableCount
                }
            }
            else {
                self.maxPickableCount = 30
            }
        }))
        
        items.append(InputBoxItem(title: "최소 선택 수", placeHolder: "1",type: .numeric, action: { count in
            if let count = count {
                if count.isEmpty {
                    self.minPickableCount = 1
                }
                else {
                    self.minPickableCount = Int(count) ??  self.minPickableCount
                }
            }
            else {
                self.minPickableCount = 1
            }
        }))
        
        items.append(CheckBoxItem(title: "선택된 친구 보기", value: true, action: {
            self.isSelectedFriendsEnabled = !self.isSelectedFriendsEnabled
            return self.isSelectedFriendsEnabled
        }))
        
        items.append(CheckBoxItem(title: "모든 데이터 보기(권한 있는 앱만 적용 됨)", value: false, action: {
            self.displayAllProfile = !self.displayAllProfile
            return self.displayAllProfile
        }))
    }
    
    func printSnapShot() {
        print("*************** SNAPSHOT *********************")
        print("isSinglePicker: \(isSinglePicker), isPopupPicker: \(isPopupPicker)")
        print("pickerTitle: \(String(describing: pickerTitle)), friendType: \(String(describing: friendType))")
        print("friendOrder: \(String(describing: friendOrder)), friendType: \(String(describing: friendOrder))")
        print("friendFilter: \(String(describing: friendFilter)), viewApperance: \(String(describing: viewApperance))")
        print("countryCode: \(String(describing: countryCodes)), friendOS: \(String(describing: friendOS))")
        print("orientation: \(String(describing: orientation)), isSearchFriendEnabled: \(isSearchFriendEnabled)")
        print("isIndexViewEnabled: \(isIndexViewEnabled)")
        print("searchPlaceHolder: \(String(describing: searchPlaceHolder)), isMyProfileEnabled: \(isMyProfileEnabled)")
        print("isFavoriteEnabled: \(isFavoriteEnabled), disabledSelectReason: \(String(describing: disabledSelectReason))")
        print("maxPickableCount: \(maxPickableCount), minPickableCount: \(minPickableCount)")
        print("isSelectedFriendsEnabled: \(isSelectedFriendsEnabled), displayAllProfile: \(displayAllProfile)")
        print("**********************************************")
    }
}

// Model
class Item {
    let title: String
    init(title: String) {
        self.title = title
    }
}

class CheckBoxItem: Item {
    var value : Bool?
    var action : (() -> Bool)?
    
    init(title: String, value: Bool? = false, action: @escaping (() -> Bool)) {
        super.init(title: title)
        
        self.value = value
        self.action = action
    }
}

class InputBoxItem: Item {
    
    enum InputBoxType{
        case string
        case numeric
    }
    
    var value: String?
    var placeHolder: String?
    var action: ((String?) -> Void)?
    var type: InputBoxType = .string
    
    init(title: String, placeHolder: String?, value: String? = "", type: InputBoxType = .string, action: @escaping ((String?) -> Void)) {
        super.init(title: title)
        
        self.value = value
        self.placeHolder = placeHolder
        self.action = action
        self.type = type
    }
}

class MenuBoxItem: Item {
    var value: String?
    var menu: [String] = []
    var action : ((String?) -> String)?
    
    init(title: String, value: String? = "NULL", menu: [String], action: @escaping ((String?) -> String)) {
        super.init(title: title)
        
        self.value = value
        self.menu = menu
        self.action = action
    }
}
